@@ -3,31 +3,41 @@
 
 """Circuits component for Action Module subscription and message handling"""
 
-import ssl
+import base64
 import json
 import logging
 import os.path
-import base64
+import ssl
+import sys
 import traceback
-from collections import Callable
 from signal import SIGINT, SIGTERM
-from six import string_types
+
+if sys.version_info.major < 3:
+    from collections import Callable
+else:
+    from collections.abc import Callable
+
 from circuits import BaseComponent, Worker
-from circuits.core.manager import ExceptionWrapper
 from circuits.core.handlers import handler
+from circuits.core.manager import ExceptionWrapper
 from requests.utils import DEFAULT_CA_BUNDLE_PATH
+from six import string_types
+
 import resilient
 from resilient import ensure_unicode
 from resilient_lib import IntegrationError
 import resilient_circuits.actions_test_component as actions_test_component
+from resilient_circuits import constants, helpers
+from resilient_circuits.action_message import (ActionMessage,
+                                               ActionMessageBase,
+                                               BaseFunctionError,
+                                               FunctionMessage, FunctionResult,
+                                               InboundMessage, StatusMessage)
 from resilient_circuits.decorators import *  # for back-compatibility, these were previously declared here
-from resilient_circuits.rest_helper import get_resilient_client, reset_resilient_client
-from resilient_circuits.action_message import ActionMessageBase, ActionMessage, \
-    FunctionMessage, InboundMessage, StatusMessage, FunctionResult, BaseFunctionError
+from resilient_circuits.rest_helper import (get_resilient_client,
+                                            reset_resilient_client)
 from resilient_circuits.stomp_component import StompClient
 from resilient_circuits.stomp_events import *
-from resilient_circuits import helpers
-from resilient_circuits import constants
 
 LOG = logging.getLogger(__name__)
 
@@ -67,7 +77,7 @@ class FunctionWorker(Worker):
            (see FallBackSignalHandler in circuits/core/helpers.py)
         """
         if signo in [SIGINT, SIGTERM]:
-            LOG.info("Worker interrupted")
+            LOG.error("Worker interrupted")
             raise SystemExit(0)
 
     @handler("task", override=True)
@@ -79,8 +89,45 @@ class FunctionWorker(Worker):
         try:
             yield result.get()
         except Exception as e:
-            LOG.error(traceback.format_exc())
-            yield ExceptionWrapper(e)
+            str_traceback = traceback.format_exc()
+            ignore_exception = False
+
+            if isinstance(self.parent, Actions):
+                app_configs = self.parent.opts
+                ignore_exception = app_configs.get(constants.APP_CONFIG_TRAP_EXCEPTION, False)
+
+            if ignore_exception:
+
+                err = str(e)
+
+                # Handle strs as unicode if Python 2
+                if sys.version_info.major == 2:
+                    err = unicode(err, "utf-8")
+                    str_traceback = unicode(str_traceback, "utf-8")
+
+                fn_name = constants.DEFAULT_UNKNOWN_STR
+
+                # args being the parameters of the decorator
+                if isinstance(args, tuple) and hasattr(args[0], "name"):
+                    fn_name = args[0].name
+
+                status_message = u"Error running '{0}'. Config '{1}' set to 'True' so ignoring exception\nERROR:\n{2}".format(fn_name, constants.APP_CONFIG_TRAP_EXCEPTION, err)
+
+                # If the loglevel is DEBUG, the full stacktrace will be added to the StatusMessage
+                if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
+                    status_message = u"{0}\n{1}".format(status_message, str_traceback)
+
+                log_message = u"{0}\n{1}".format(status_message, str_traceback)
+
+                yield StatusMessage(status_message)
+
+                LOG.warning(log_message)
+
+                yield FunctionResult({"success": False, "reason": err})
+
+            else:
+                LOG.error(str_traceback)
+                yield ExceptionWrapper(e)
 
 
 class ResilientComponent(BaseComponent):
@@ -91,6 +138,7 @@ class ResilientComponent(BaseComponent):
        and actions and functions will be dispatched to its `handler` and `function` methods.
     """
     test_mode = False  # True with --test-actions option
+    IS_SELFTEST = False
 
     def __init__(self, opts):
         super(ResilientComponent, self).__init__()
@@ -333,6 +381,7 @@ class Actions(ResilientComponent):
         self._configure_opts(opts)
         self.max_retry_count = int(opts.get("stomp_max_retries")) # default from app.py:DEFAULT_STOMP_MAX_RETRIES
         self.stomp_timeout = int(opts.get("stomp_timeout", SUBSCRIBE_TO_QUEUES_TIMEOUT))
+        self.heartbeat_timeouts = []
 
         timer_internal = int(opts['resilient'].get("stomp_timer_interval", RETRY_TIMER_INTERVAL))
 
@@ -402,6 +451,10 @@ class Actions(ResilientComponent):
     def on_stomp_connected(self):
         """Client has connected to the STOMP server"""
         LOG.info("STOMP connected.")
+
+        # Once STOMP has connected, we want to empty the heartbeat_timeouts list
+        self.heartbeat_timeouts = []
+
         # Stop retrying the failed deliveries from previous session
         # We'll ack them right away if they are re-delivered
         # TODO: We should age these out of here at some point. 24 hours?
@@ -411,10 +464,43 @@ class Actions(ResilientComponent):
             self._resilient_ack_delivery_failures[key]["from_prev_conn"] = True
 
     @handler("HeartbeatTimeout")
-    def on_heartbeat_timeout(self):
-        """Heartbeat timed out from the STOMP server"""
-        LOG.error("Reconnecting after STOMP heartbeat timeout.")
-        # Disconnect and Reconnect
+    def on_heartbeat_timeout(self, event):
+        """
+        Handler for a HeartbeatTimeout event
+
+        Checks to see if the 'heartbeat_timeout_threshold' config is set. Keeps
+        track of HBs in a list. Each HB has a ts value. If the difference
+        between the first and current HB's ts is greater than the config,
+        resilient-circuits exits with an exit code of 34
+        """
+
+        heartbeat_timeout_threshold = self.opts.get(constants.APP_CONFIG_HEARTBEAT_TIMEOUT_THRESHOLD)
+
+        # If heartbeat_timeout_threshold config is set
+        if heartbeat_timeout_threshold:
+
+            # Add the current HB to the list
+            self.heartbeat_timeouts.append(event)
+
+            # If more than 1 HBs are in the list
+            if len(self.heartbeat_timeouts) > 1:
+
+                # Remove any invalid timeouts and sort the list
+                self.heartbeat_timeouts = helpers.filter_heartbeat_timeout_events(self.heartbeat_timeouts)
+
+                # Get the difference in seconds between the oldest and current timeout
+                delta = int(self.heartbeat_timeouts[-1].ts - self.heartbeat_timeouts[0].ts)
+
+                # If the delta is greater than the config, log an error and exit resilient-circuits
+                if delta > heartbeat_timeout_threshold:
+                    LOG.error("'%s' is set to '%ss' and '%ss' have passed since the first HeartbeatTimeout, exiting...",
+                              constants.APP_CONFIG_HEARTBEAT_TIMEOUT_THRESHOLD, heartbeat_timeout_threshold, delta)
+
+                    sys.exit(constants.EXIT_STOMP_HEARTBEAT_TIMEOUT)
+
+        LOG.error("Trying to reconnect after STOMP HeartbeatTimeout")
+
+        # Fire a Disconnect event to Reconnect
         self.fire(Disconnect(flush=False, reconnect=True))
 
     @handler("OnStompError")
@@ -1142,4 +1228,4 @@ class Actions(ResilientComponent):
                         self.fire(result, '*')
                     # Send a message back to the test client
                     self.fire(Event.create("test_response", fevent.test_msg_id, reply_message), '*')
-                    LOG.debug("Test Action: No ack done.")
\ No newline at end of file
+                    LOG.debug("Test Action: No ack done.")
