@@ -4,32 +4,36 @@
 
 import logging
 import os
-from threading import Thread
 import time
 from collections import defaultdict
+from threading import Thread
+
 import pkg_resources
 from requests.exceptions import ConnectionError, SSLError
+from resilient import BasicHTTPException, SimpleClient, SimpleHTTPException
 from resilient import constants as res_constants
-from resilient import BasicHTTPException, SimpleHTTPException, SimpleClient, is_env_proxies_set, get_and_parse_proxy_env_var
-from resilient_circuits.actions_component import SELFTEST_ERRORS, SELFTEST_SUBSCRIPTIONS
+from resilient import get_and_parse_proxy_env_var, is_env_proxies_set
+from resilient_circuits import app, constants, helpers
+from resilient_circuits.actions_component import (SELFTEST_ERRORS,
+                                                  SELFTEST_SUBSCRIPTIONS)
 from resilient_circuits.stomp_events import SelftestTerminateEvent
-from resilient_circuits import constants, helpers, app
 
 # Get the same logger object that is used for resilient_circuits commands
 LOG = logging.getLogger(constants.CMDS_LOGGER_NAME)
 
 ERROR_EXIT_CODES_MAP = {
-    1: 1,       # Error running App's selftest
-    20: 20,     # REST: Generic connection error
-    401: 21,    # REST: Connection unauthorized
-    22: 22,     # REST: OSError (Could not find Certificate file)
-    23: 23,     # REST: SSL Error (Invalid Certificate Error)
-    24: 24,     # REST: Organization Membership Error
-    25: 25,     # REST: Invalid Username or Password
-    30: 30,     # STOMP: Generic connection error
-    31: 31,     # STOMP: Not authorized to instansiate STOMP connection
-    32: 32,     # STOMP: Not authorized to read from queue
-    33: 33      # STOMP: Timed out trying to see if resilient-circuits is subscribed to a message destination
+    1: constants.EXIT_SELFTEST_ERROR,
+    2: constants.EXIT_SELFTEST_UNIMPLEMENTED,
+    20: constants.EXIT_REST_CONNECTION_ERROR,
+    401: constants.EXIT_REST_UNAUTHORIZED,
+    22: constants.EXIT_REST_OS_ERROR,
+    23: constants.EXIT_REST_SSL_ERROR,
+    24: constants.EXIT_REST_ORG_ERROR,
+    25: constants.EXIT_REST_INVALID_PW,
+    30: constants.EXIT_STOMP_ERROR,
+    31: constants.EXIT_STOMP_UNAUTHORIZED_CONN,
+    32: constants.EXIT_STOMP_UNAUTHORIZED_Q,
+    33: constants.EXIT_STOMP_Q_TIMEOUT
 }
 
 
@@ -166,7 +170,7 @@ def check_soar_stomp_connection(cmd_line_args, app_configs):
             LOG.parent.info("- Waiting for subscription to message destination. Sleeping for 2 seconds")
             time.sleep(2)
 
-            if helpers.should_timeout(start_time, app_configs.get(constants.DEFAULT_SELFTEST_TIMEOUT_KEY, constants.DEFAULT_SELFTEST_TIMEOUT_VALUE)):
+            if helpers.should_timeout(start_time, app_configs.get(constants.APP_CONFIG_SELFTEST_TIMEOUT, constants.DEFAULT_SELFTEST_TIMEOUT_VALUE)):
                 resilient_circuits_instance.action_component.fire(SelftestTerminateEvent())
                 error_connecting_to_soar(host, "Could not subscribe to any message destinations", 33)
 
@@ -221,6 +225,9 @@ def run_apps_selftest(cmd_line_args, app_configs):
 
     # Prepare a count of exceptions found with selftests.
     selftest_failure_count = 0
+    selftest_unimplemented_count = 0
+    selftest_not_found_count = 0
+    selftest_unknown_count = 0
 
     for dist, component_list in components.items():
         if cmd_line_args.install_list is None or dist.project_name in install_list:
@@ -233,7 +240,11 @@ def run_apps_selftest(cmd_line_args, app_configs):
             LOG.info("\n%s: ", dist.project_name)
             for ep in component_list:
                 # load the entry point
-                f_selftest = ep.load()
+                try:
+                    f_selftest = ep.load()
+                except ModuleNotFoundError:
+                    selftest_not_found_count += 1
+                    continue
 
                 try:
                     # f_selftest is the selftest function, we pass the selftest resilient options in case it wants to use it
@@ -251,8 +262,12 @@ def run_apps_selftest(cmd_line_args, app_configs):
                     if isinstance(state, str):
                         LOG.info("\t%s: %s\n\tselftest output:\n\t%s\n\tElapsed time: %f seconds", ep.name, state, status, delta_seconds)
 
-                        if state.lower() == "failure":
+                        if state.lower() == constants.SELFTEST_FAILURE_STATE:
                             selftest_failure_count += 1
+                        elif state.lower() == constants.SELFTEST_UNIMPLEMENTED_STATE:
+                            selftest_unimplemented_count += 1
+                        elif state.lower() != constants.SELFTEST_SUCCESS_STATE:
+                            selftest_unknown_count += 1
 
                     else:
                         LOG.info("\t%s:\n\tUnsupported dictionary returned:\n\t%s\n\tElapsed time: %f seconds", ep.name, status, delta_seconds)
@@ -263,15 +278,22 @@ def run_apps_selftest(cmd_line_args, app_configs):
                     continue
 
     # any missed packages?
-    if len(install_list):
-        LOG.warning("%s not found. Check package name(s)", install_list)
+    if install_list:
+        LOG.warning("\n%s not found. Selftest not ran for %s. Check package name(s)\n", install_list, install_list)
 
     # Check if any failures were found and printed to the console
-    if selftest_failure_count:
+    if selftest_failure_count or selftest_unknown_count:
         LOG.info("\nERROR: running selftest for App.\nError Code: {0}".format(ERROR_EXIT_CODES_MAP.get(1, 1)))
         exit(ERROR_EXIT_CODES_MAP.get(1, 1))
+    elif selftest_unimplemented_count:
+        LOG.info("\nERROR: selftest is unimplemented for this App. Note: the App may still continue to work...\nError Code: {0}".format(ERROR_EXIT_CODES_MAP.get(2, 2)))
+        exit(ERROR_EXIT_CODES_MAP.get(2, 2))
+    elif selftest_not_found_count:
+        LOG.info("\nERROR: selftest not found for this App. Note: the App may still continue to work...\nError Code: {0}".format(ERROR_EXIT_CODES_MAP.get(2, 2)))
+        exit(ERROR_EXIT_CODES_MAP.get(2, 2))
 
-    LOG.info("{0}Successfully ran App's selftest!{0}".format(constants.LOG_DIVIDER))
+    if not install_list:
+        LOG.info("{0}Successfully ran App's selftest!{0}".format(constants.LOG_DIVIDER))
 
 
 def execute_command(cmd_line_args):
