@@ -198,11 +198,13 @@ def get_packages(working_set):
 
     :param working_set: the working_set for all packages installed in this env
     :type working_set: setuptools.pkg_resources.WorkingSet obj
-    :return: pkg_list: a list of tuples [('name','version')] e.g. [('resilient-circuits', '39.0.0')]
+    :return: pkg_list: a list of tuples [('name','version')] e.g. [('resilient-circuits', '39.0.0')].
+        If ``working_set`` is not a ``pkg_resources.WorkingSet`` object, just return the current value of ``working_set``
     :rtype: list
     """
 
-    isinstance(working_set, pkg_resources.WorkingSet)
+    if not isinstance(working_set, pkg_resources.WorkingSet):
+        return working_set
 
     pkg_list = []
 
@@ -243,56 +245,6 @@ def get_env_str(packages):
     return env_str
 
 
-def remove_tag(original_res_obj):
-    """
-    Return the original_res_obj with any of the "tags"
-    attribute set to an empty list
-
-    Example:
-    ```
-    mock_res_obj = {
-        "tags": [{"tag_handle": "fn_tag_test", "value": None}],
-        "functions": [
-            {"export_key": "fn_tag_test_function",
-            "tags": [{'tag_handle': 'fn_tag_test', 'value': None}]}
-        ]
-    }
-
-    new_res_obj = remove_tag(mock_res_obj)
-
-    Returns: {
-        "tags": [],
-        "functions": [
-            {"export_key": "fn_tag_test_function", "tags": []}
-        ]
-    }
-    ```
-    :param original_res_obj: the res_obj you want to remove the tags attribute from
-    :type original_res_obj: dict
-    :return: new_res_obj: a dict with the tag attribute removed
-    :rtype: dict
-    """
-    ATTRIBUTE_NAME = "tags"
-
-    new_res_obj = copy.deepcopy(original_res_obj)
-
-    if isinstance(new_res_obj, dict):
-
-        # Set "tags" to empty list
-        if new_res_obj.get(ATTRIBUTE_NAME):
-            new_res_obj[ATTRIBUTE_NAME] = []
-
-        # Recursively loop the dict
-        for obj_name, obj_value in new_res_obj.items():
-
-            if isinstance(obj_value, list):
-                for index, obj in enumerate(obj_value):
-                    new_res_obj[obj_name][index] = remove_tag(obj)
-
-            elif isinstance(obj_value, dict):
-                new_res_obj[obj_name] = remove_tag(obj_value)
-
-    return new_res_obj
 
 
 def get_queue(destination):
@@ -325,14 +277,14 @@ def get_queue(destination):
 
 def is_this_a_selftest(component):
     """
-    Return a True or False if this instantiation of
-    resilient-circuits is from selftest or not.
+    Return ``True`` or ``False`` if this instantiation of
+    ``resilient-circuits`` is from selftest or not.
 
-    :param component: the current component that is calling this method (usually 'self')
+    :param component: the current component that is calling this method (usually ``self``)
     :type component: circuits.Component
     :rtype: bool
     """
-    return bool(component.parent.name == "App" and component.parent.IS_SELFTEST)
+    return component.IS_SELFTEST
 
 
 def should_timeout(start_time, timeout_value):
@@ -373,3 +325,26 @@ def get_user(app_configs):
             return None
 
     return usr
+
+
+def filter_heartbeat_timeout_events(heartbeat_timeouts):
+    """
+    Sort a list of HeartbeatTimeout on their ts and
+    remove an occurrences of HeartbeatTimeout whose ts
+    value is -1
+
+    :param heartbeat_timeouts: List of HeartbeatTimeout
+    :type heartbeat_timeouts: [HeartbeatTimeout]
+    :return: Sorted and filtered list of HeartbeatTimeout
+    :rtype: [HeartbeatTimeout]
+    """
+
+    if not heartbeat_timeouts:
+        return heartbeat_timeouts
+
+    # Remove any elements whose ts is -1
+    heartbeat_timeouts = [hb for hb in heartbeat_timeouts if hb.ts != -1]
+
+    heartbeat_timeouts.sort()
+
+    return heartbeat_timeouts
